""" Converting Astropy Table Object into Pandas Dataframe """
# This module was developed with funding from 
# Google Summer of Code 2015
# author - Ankit Kumar  <ankitkmr.iitk@gmail.com>

from pandas import DataFrame

from astropy.table import Table, Column
from astropy.utils import OrderedDict
from astropy.table.column import MaskedColumn

def _to_pandas(astropyTable):
	"""
	Return a :class:`pandas.DataFrame` instance

	Returns
	-------
	dataframe : :class:`pandas.DataFrame`
		A pandas :class:`pandas.DataFrame` instance

	Raises
	------
	ImportError
		If pandas is not installed
	ValueError
		If the Table contains mixin or multi-dimensional columns
	"""
	from pandas import DataFrame

	if astropyTable.has_mixin_columns:
		raise ValueError("Cannot convert a table with mixin columns to a pandas DataFrame")

	if any(getattr(col, 'ndim', 1) > 1 for col in astropyTable.columns.values()):
		raise ValueError("Cannot convert a table with multi-dimensional columns to a pandas DataFrame")

	out = OrderedDict()

	for name, column in astropyTable.columns.items():
		if isinstance(column, MaskedColumn):
			if column.dtype.kind in ['i', 'u']:
				out[name] = column.astype(float).filled(np.nan)
			elif column.dtype.kind in ['f', 'c']:
				out[name] = column.filled(np.nan)
			else:
				out[name] = column.astype(np.object).filled(np.nan)
		else:
			out[name] = column

		if out[name].dtype.byteorder not in ('=', '|'):
			out[name] = out[name].byteswap().newbyteorder()

	return DataFrame(out)




from __future__ import absolute_import, division, print_function

import sys
from sunpy.extern.six.moves import range

__all__ = ['TTYProgressBar']

class TTYProgressBar(object):
    """
    A simple progress bar to visualize progress on a TTY (teletypewriter).
    It needs to support '\b' to delete characters.

    The ProgressBar interface is start, finish, draw and poke.
    """
    SYMBOL = '='
    LEFT_BORDER = '['
    RIGHT_BORDER = ']'

    def __init__(self, n, current=0, width=40, output=sys.stdout):
        """
        Parameters
        ----------
        n : int
            Total number of items until completion
        current : int
            Current state of completion
        width : int
            Width of progress bar.
        output : file
            Teletypewriter to print on.
        """
        self.n = n
        self.current = current
        self.width = width
        # Future division, so it is float.
        self.step = self.n / self.width
        self.output = output

    def start(self):
        """
        Draw empty bar to output.
        """
        self.output.write(
            self.LEFT_BORDER + " " * (len(self.SYMBOL) * self.width) +
                self.RIGHT_BORDER
        )
        self.output.flush()
        self.output.write("\b" * (self.width+len(self.RIGHT_BORDER)))

    def finish(self):
        """
        Finish the bar, the ProgressBar cannot be used after this
        method was called.
        """
        print()

    def _draw_one(self):
        """
        Advance progress bar by one.
        """
        self.output.write(self.SYMBOL)

    def draw(self):
        """
        Draw current state of progress bar onto and empty bar.
        """
        cur = self.current
        self.current = 0
        for _ in range(cur):
            self.poke()

    def poke(self, n=1):
        """
        Increase finished items by n. May advance the progress bar by one
        or more fields.
        """
        if self.current > self.n:
            raise ValueError("ProgressBar overflowed.")

        diff = int((self.current + n) / self.step) - int(self.current / self.step)
        for _ in range(diff):
            self._draw_one()
        self.current += n



"""
SunPy specific representations.

This submodule extends `astropy.coordinates.representations` with two Spherical
representation classes, primarily for use with the Helioprojective coordinate
system, due to the convention of Longitude going from -180 to 180 degrees.
"""

from __future__ import absolute_import, division

from collections import OrderedDict

from astropy import units as u
from astropy.coordinates.representation import (SphericalRepresentation,
                                                UnitSphericalRepresentation,
                                                CartesianRepresentation)
from astropy.coordinates import Longitude, Latitude

__all__ = ['Longitude180', 'SphericalWrap180Representation',
           'UnitSphericalWrap180Representation']


class Longitude180(Longitude):
    """
    Quantity class that represents Longitude.
    It sets the default wrap_angle to 180 degrees.

    Parameters
    ----------
    angle : array, list, scalar, `~astropy.units.Quantity`,
       :class:`~astropy.coordinates.Angle` The angle value(s). If a tuple,
       will be interpreted as ``(h, m s)`` or ``(d, m, s)`` depending
       on ``unit``. If a string, it will be interpreted following the
       rules described for :class:`~astropy.coordinates.Angle`.

       If ``angle`` is a sequence or array of strings, the resulting
       values will be in the given ``unit``, or if `None` is provided,
       the unit will be taken from the first given value.

    unit : :class:`~astropy.units.UnitBase`, str, optional
       The unit of the value specified for the angle.  This may be
       any string that `~astropy.units.Unit` understands, but it is
       better to give an actual unit object.  Must be an angular
       unit.

    wrap_angle : :class:`~astropy.coordinates.Angle` or equivalent, or None
       Angle at which to wrap back to ``wrap_angle - 180 deg``.
       If ``None`` (default), it will be taken to be 180 deg unless ``angle``
       has a ``wrap_angle`` attribute already (i.e., is a ``Longitude``),
       in which case it will be taken from there.
    """

    def __new__(cls, angle, unit=None, wrap_angle=180 * u.deg, **kwargs):
        self = super(Longitude180, cls).__new__(cls,
                                                angle,
                                                unit=unit,
                                                wrap_angle=wrap_angle,
                                                **kwargs)
        return self


class UnitSphericalWrap180Representation(UnitSphericalRepresentation):
    """
    Representation of points in 3D Spherical coordinates.
    This representation allows for a negative Longitude.
    It does so by setting wrap_angle=180 degrees.

    Parameters
    ----------
    lon, lat: `~astropy.units.Quantity`
        The longitude and latitude of the point(s) in angular units. The
        latitude should be between -90 and +90 degrees, and the longitude
        is allowed to have any value between -180 to 180 degrees. These
        can also be instances of `~astropy.units.Angle`,
        `~astropy.coordinates.Longitude`, or `~astropy.coordinates.Latitude`.

    distance: `~astropy.units.Quantity`
        The distance to the point(s). If the distance is a length, it is
        passed to the :class:`~astropy.coordinates.Distance` class, else
        it is passed to the :class:`~astropy.units.Quantity` class.

    copy: bool, optional
        If True, arrays will be copied rather than referenced.
    """

    attr_classes = OrderedDict([('lon', Longitude180), ('lat', Latitude)])
    recommended_units = {'lon': u.deg, 'lat': u.deg}


class SphericalWrap180Representation(SphericalRepresentation):
    """
    Representation of points in 3D Spherical coordinates.
    This representation allows for a negative Longitude.
    It does so by setting wrap_angle=180 degrees.

    Parameters
    ----------
    lon, lat: `~astropy.units.Quantity`
        The longitude and latitude of the point(s) in angular units. The
        latitude should be between -90 and +90 degrees, and the longitude
        is allowed to have any value between -180 to 180 degrees. These
        can also be instances of `~astropy.units.Angle`,
        `~astropy.coordinates.Longitude`, or `~astropy.coordinates.Latitude`.

    distance: `~astropy.units.Quantity`
        The distance to the point(s). If the distance is a length, it is
        passed to the :class:`~astropy.coordinates.Distance` class, else
        it is passed to the :class:`~astropy.units.Quantity` class.

    copy: bool, optional
        If True, arrays will be copied rather than referenced.
    """

    attr_classes = OrderedDict([('lon', Longitude180), ('lat', Latitude),
                                ('distance', u.Quantity)])
    recommended_units = {'lon': u.deg, 'lat': u.deg}

    _unitrep = UnitSphericalWrap180Representation

